from restic.internal import command_executor


class List:

    def __init__(self, restic_base_command):
        self.base_command = restic_base_command

    def locks(self):
        cmd = self.base_command() + ['list', 'locks']
        return self.run(cmd)

    def run(self, cmd):
        result = command_executor.execute(cmd)

        return result.splitlines()



import json

import restic.errors
from restic.internal import command_executor


def run(restic_base_command,
        dry_run=None,
        group_by=None,
        tags=None,
        host=None,
        keep_last=None,
        keep_hourly=None,
        keep_daily=None,
        keep_weekly=None,
        keep_monthly=None,
        keep_yearly=None,
        keep_within=None,
        prune=False):
    cmd = restic_base_command + ['forget']

    if dry_run:
        cmd.extend(['--dry-run'])

    if group_by:
        cmd.extend(['--group-by', group_by])

    if tags:
        for tag in tags:
            cmd.extend(['--tag', tag])

    if host:
        cmd.extend(['--host', host])

    if keep_last:
        cmd.extend(['--keep-last', str(keep_last)])

    if keep_hourly:
        cmd.extend(['--keep-hourly', str(keep_hourly)])

    if keep_daily:
        cmd.extend(['--keep-daily', str(keep_daily)])

    if keep_weekly:
        cmd.extend(['--keep-weekly', str(keep_weekly)])

    if keep_monthly:
        cmd.extend(['--keep-monthly', str(keep_monthly)])

    if keep_yearly:
        cmd.extend(['--keep-yearly', str(keep_yearly)])

    if keep_within:
        cmd.extend(['--keep-within', str(keep_within)])

    if prune:
        cmd.extend(['--prune'])

    return _parse_result(command_executor.execute(cmd))


def _parse_result(result):
    # The result is JSON followed by 0 or more non-JSON lines.
    result_lines = result.split('\n')
    try:
        # forget result can be empty
        if result_lines[0]:
            return json.loads(result_lines[0])
        return {}
    except json.decoder.JSONDecodeError as e:
        raise restic.errors.UnexpectedResticResponse(
            'Unexpected result from restic. Expected JSON, got: ' +
            result_lines[0]) from e



import json

from restic.internal import command_executor


class Key:

    def __init__(self, restic_base_command):
        self.base_command = restic_base_command

    def list(self):
        cmd = self.base_command() + ['key', 'list']

        result = command_executor.execute(cmd)
        return json.loads(result)

    def add(self, host=None, new_password_file=None, user=None):
        cmd = self.base_command() + ['key', 'add']

        if host:
            cmd.extend(['--host', host])

        if new_password_file:
            cmd.extend(['--new-password-file', new_password_file])

        if user:
            cmd.extend(['--user', user])

        return command_executor.execute(cmd)

    def remove(self, key_id):
        cmd = self.base_command() + ['key', 'remove', key_id]

        return command_executor.execute(cmd)

    def passwd(self, new_password_file=None):
        cmd = self.base_command() + ['key', 'passwd']

        if new_password_file:
            cmd.extend(['--new-password-file', new_password_file])

        return command_executor.execute(cmd)



from restic.internal import command_executor


def run(restic_base_command,
        exclude=None,
        exclude_file=None,
        forget=False,
        dry_run=False,
        snapshot_id=None):
    cmd = restic_base_command + ['rewrite']

    if exclude:
        for exclude_path in exclude:
            cmd.extend(['--exclude', exclude_path])

    if exclude_file:
        cmd.extend(['--exclude-file', exclude_file])

    if forget:
        cmd.append('--forget')

    if dry_run:
        cmd.append('--dry-run')

    if snapshot_id:
        cmd.append(snapshot_id)

    return command_executor.execute(cmd)



import json
import re

from restic.internal import command_executor


def run(restic_base_command,
        copy_chunker_params=False,
        from_repo=None,
        from_password_file=None):
    cmd = restic_base_command + ['init']

    if copy_chunker_params:
        cmd.append('--copy-chunker-params')

    if from_repo:
        cmd.extend(['--from-repo', from_repo])

    if from_password_file:
        cmd.extend(['--from-password-file', from_password_file])

    return _parse_result(command_executor.execute(cmd))


def _parse_result(result):
    try:
        return json.loads(result)['id']
    except json.decoder.JSONDecodeError:
        return _parse_plaintext_result(result)


def _parse_plaintext_result(result):
    """Parse legacy plaintext result.

    Prior to restic 0.15.0, restic returned a plaintext response even when the
    caller specified --json.
    """
    return re.match(r'created restic repository ([a-z0-9]+) at .+',
                    result).group(1)



import unittest
from unittest import mock

import restic
from restic.internal import rewrite


class RewriteTest(unittest.TestCase):

    @mock.patch.object(rewrite.command_executor, 'execute')
    def test_rewrite_single_path(self, mock_execute):
        restic.rewrite(exclude=['/foo/bar'])
        mock_execute.assert_called_with(
            ['restic', '--json', 'rewrite', '--exclude', '/foo/bar'])

    @mock.patch.object(rewrite.command_executor, 'execute')
    def test_rewrite_multiple_paths(self, mock_execute):
        restic.rewrite(exclude=['/foo/bar', '/biz/baz'])
        mock_execute.assert_called_with([
            'restic', '--json', 'rewrite', '--exclude', '/foo/bar', '--exclude',
            '/biz/baz'
        ])

    @mock.patch.object(rewrite.command_executor, 'execute')
    def test_rewrite_specific_snapshot_id(self, mock_execute):
        restic.rewrite(exclude=['/foo/bar'], snapshot_id='dummy-snapshot-id')
        mock_execute.assert_called_with([
            'restic', '--json', 'rewrite', '--exclude', '/foo/bar',
            'dummy-snapshot-id'
        ])

    @mock.patch.object(rewrite.command_executor, 'execute')
    def test_rewrite_and_forget_specific_snapshot_id(self, mock_execute):
        restic.rewrite(exclude=['/foo/bar'],
                       forget=True,
                       snapshot_id='dummy-snapshot-id')
        mock_execute.assert_called_with([
            'restic', '--json', 'rewrite', '--exclude', '/foo/bar', '--forget',
            'dummy-snapshot-id'
        ])

    @mock.patch.object(rewrite.command_executor, 'execute')
    def test_rewrite_from_exclude_file(self, mock_execute):
        restic.rewrite(exclude_file='/data/excludes.txt')
        mock_execute.assert_called_with([
            'restic', '--json', 'rewrite', '--exclude-file',
            '/data/excludes.txt'
        ])

    @mock.patch.object(rewrite.command_executor, 'execute')
    def test_rewrite_as_dry_run(self, mock_execute):
        restic.rewrite(exclude=['/foo/bar'],
                       dry_run=True,
                       snapshot_id='dummy-snapshot-id')
        mock_execute.assert_called_with([
            'restic',
            '--json',
            'rewrite',
            '--exclude',
            '/foo/bar',
            '--dry-run',
            'dummy-snapshot-id',
        ])



import unittest
from unittest import mock

import restic
from restic.internal import snapshots


class SnapshotsTest(unittest.TestCase):

    def setUp(self):
        self.maxDiff = None

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_simple(self, mock_execute):
        mock_execute.return_value = '[]'

        self.assertEqual([], restic.snapshots())

        mock_execute.assert_called_with(['restic', '--json', 'snapshots'])

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_group_by_host(self, mock_execute):
        mock_execute.return_value = '[]'

        self.assertEqual([], restic.snapshots(group_by='host'))

        mock_execute.assert_called_with(
            ['restic', '--json', 'snapshots', '--group-by', 'host'])

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_id(self, mock_execute):
        mock_execute.return_value = '[]'

        self.assertEqual([], restic.snapshots(snapshot_id='latest'))

        mock_execute.assert_called_with(
            ['restic', '--json', 'snapshots', 'latest'])

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_tags(self, mock_execute):
        mock_execute.return_value = '[]'

        self.assertEqual([], restic.snapshots(tags=['test', 'test2']))

        mock_execute.assert_called_with(
            ['restic', '--json', 'snapshots', '--tag', 'test,test2'])

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_parses_result_json(self, mock_execute):
        mock_execute.return_value = """
[
  {
    "group_key": {
      "hostname": "ace809d23440",
      "paths": null,
      "tags": null
    },
    "snapshots": [
      {
        "gid": 3434,
        "hostname": "ace809d23440",
        "id": "bbe7f04941ed969ee940bb41dc04196027fcfb83dbdfea93c16afb2cb9f6dd81",
        "paths": [
          "/tmp/tmp6594jneh/mydata.txt"
        ],
        "short_id": "bbe7f049",
        "time": "2021-03-28T21:31:44.477122573Z",
        "tree": "f589421bafdae95f5be5eea6285074b7ddc54aa0ffd1ad606f74d1e6207d20a3",
        "uid": 3434,
        "username": "circleci"
      }
    ]
  }
]
""".strip()

        # Ignore complaint about too long of a line.
        # pylint: disable=C0301
        self.assertEqual([{
            'group_key': {
                'hostname': 'ace809d23440',
                'paths': None,
                'tags': None
            },
            'snapshots': [{
                'time':
                    '2021-03-28T21:31:44.477122573Z',
                'tree':
                    'f589421bafdae95f5be5eea6285074b7ddc54aa0ffd1ad606f74d1e6207d20a3',
                'paths': ['/tmp/tmp6594jneh/mydata.txt'],
                'hostname':
                    'ace809d23440',
                'username':
                    'circleci',
                'uid':
                    3434,
                'gid':
                    3434,
                'id':
                    'bbe7f04941ed969ee940bb41dc04196027fcfb83dbdfea93c16afb2cb9f6dd81',
                'short_id':
                    'bbe7f049'
            }]
        }], restic.snapshots())

    @mock.patch.object(snapshots.command_executor, 'execute')
    def test_snapshots_parses_multiple_snapshots_json(self, mock_execute):
        mock_execute.return_value = """
[
    {
        "group_key": {
            "hostname": "3099758ccbd9",
            "paths": null,
            "tags": null
        },
        "snapshots": [
            {
                "time": "2023-12-31T20:39:22.449822393Z",
                "tree": "d4aed49b16b1fcd2ebcfbc8cfd7c53bcff2397b8673b6764dd6411223775d6b2",
                "paths": [
                    "/tmp/tmpuvf9pfam/mydata.txt"
                ],
                "hostname": "3099758ccbd9",
                "username": "circleci",
                "uid": 1001,
                "gid": 1002,
                "program_version": "restic 0.16.2",
                "id": "7540e94d8117078bb26b0e4a85dd353a41665e084d3771f04e5f72f6b0f24a6f",
                "short_id": "7540e94d"
            },
            {
                "time": "2023-12-31T20:39:25.811155387Z",
                "parent": "7540e94d8117078bb26b0e4a85dd353a41665e084d3771f04e5f72f6b0f24a6f",
                "tree": "a39c45c1b202985f766a177a663a5e4f37acd5bfa9bd324fb4eaf62a3c3e0dea",
                "paths": [
                    "/tmp/tmpuvf9pfam/mydata.txt"
                ],
                "hostname": "3099758ccbd9",
                "username": "circleci",
                "uid": 1001,
                "gid": 1002,
                "program_version": "restic 0.16.2",
                "id": "57d3496886edf2cb0e808838734dc108fb1d929ae80cafdb68597e75702aecbb",
                "short_id": "57d34968"
            }
        ]
    }
]
""".strip()

        # Ignore complaint about too long of a line.
        # pylint: disable=C0301
        self.assertEqual([{
            'group_key': {
                'hostname': '3099758ccbd9',
                'paths': None,
                'tags': None
            },
            'snapshots': [{
                'time':
                    '2023-12-31T20:39:22.449822393Z',
                'tree':
                    'd4aed49b16b1fcd2ebcfbc8cfd7c53bcff2397b8673b6764dd6411223775d6b2',
                'paths': ['/tmp/tmpuvf9pfam/mydata.txt'],
                'hostname':
                    '3099758ccbd9',
                'username':
                    'circleci',
                'uid':
                    1001,
                'gid':
                    1002,
                'program_version':
                    'restic 0.16.2',
                'id':
                    '7540e94d8117078bb26b0e4a85dd353a41665e084d3771f04e5f72f6b0f24a6f',
                'short_id':
                    '7540e94d'
            }, {
                'time':
                    '2023-12-31T20:39:25.811155387Z',
                'tree':
                    'a39c45c1b202985f766a177a663a5e4f37acd5bfa9bd324fb4eaf62a3c3e0dea',
                'paths': ['/tmp/tmpuvf9pfam/mydata.txt'],
                'hostname':
                    '3099758ccbd9',
                'username':
                    'circleci',
                'uid':
                    1001,
                'gid':
                    1002,
                'parent':
                    '7540e94d8117078bb26b0e4a85dd353a41665e084d3771f04e5f72f6b0f24a6f',
                'program_version':
                    'restic 0.16.2',
                'id':
                    '57d3496886edf2cb0e808838734dc108fb1d929ae80cafdb68597e75702aecbb',
                'short_id':
                    '57d34968'
            }]
        }], restic.snapshots())



import json

from restic.internal import command_executor


class Cat:

    def __init__(self, restic_base_command):
        self.base_command = restic_base_command

    def masterkey(self):
        cmd = self.base_command() + ['cat', 'masterkey']
        return self.run(cmd)

    def config(self):
        cmd = self.base_command() + ['cat', 'config']
        return self.run(cmd)

    def run(self, cmd):
        return json.loads(command_executor.execute(cmd))



import unittest
from unittest import mock

import restic
from restic.internal import cat


class ListLocksTest(unittest.TestCase):

    @mock.patch.object(cat.command_executor, 'execute')
    def test_list_locks(self, mock_execute):
        mock_execute.return_value = """
e3a938731770e741fe34fb43fc26ff3088e77ba3ebb0792181a72e0fdb5fbe0d
2bb62a61dfd1853f845d9baf4330f1af57172516260ccb45989569f459f73753
""".strip()

        result = restic.list.locks()

        mock_execute.assert_called_with([
            'restic',
            '--json',
            'list',
            'locks',
        ])

        self.assertEqual([
            'e3a938731770e741fe34fb43fc26ff3088e77ba3ebb0792181a72e0fdb5fbe0d',
            '2bb62a61dfd1853f845d9baf4330f1af57172516260ccb45989569f459f73753'
        ], result)



import unittest
from unittest import mock

import restic
from restic.internal import key


class KeyListTest(unittest.TestCase):

    @mock.patch.object(key.command_executor, 'execute')
    def test_key_list(self, mock_execute):
        mock_execute.return_value = """
           [{
             "current":true,
             "id":"f1fe5fc6",
             "userName":"user1",
             "hostName":"example",
             "created":"2022-12-02 11:25:13"
            }]"""

        key_result = restic.key.list()

        mock_execute.assert_called_with(['restic', '--json', 'key', 'list'])

        self.assertEqual([{
            'current': True,
            'id': 'f1fe5fc6',
            'userName': 'user1',
            'hostName': 'example',
            'created': '2022-12-02 11:25:13',
        }], key_result)


class KeyAddTest(unittest.TestCase):

    @mock.patch.object(key.command_executor, 'execute')
    def test_key_add(self, mock_execute):
        restic.key.add(new_password_file='/path/to/new-password-file')

        mock_execute.assert_called_with([
            'restic', '--json', 'key', 'add', '--new-password-file',
            '/path/to/new-password-file'
        ])


class KeyPasswdTest(unittest.TestCase):

    @mock.patch.object(key.command_executor, 'execute')
    def test_key_passwd(self, mock_execute):
        restic.key.passwd(new_password_file='/path/to/new-password-file')

        mock_execute.assert_called_with([
            'restic', '--json', 'key', 'passwd', '--new-password-file',
            '/path/to/new-password-file'
        ])


class KeyRemoveTest(unittest.TestCase):

    @mock.patch.object(key.command_executor, 'execute')
    def test_key_remove(self, mock_execute):
        restic.key.remove(key_id='abc123')

        mock_execute.assert_called_with(
            ['restic', '--json', 'key', 'remove', 'abc123'])



import unittest
from unittest import mock

import restic
from restic.internal import cat


class CatMasterkeyTest(unittest.TestCase):

    @mock.patch.object(cat.command_executor, 'execute')
    def test_cat_masterkey(self, mock_execute):
        mock_execute.return_value = """{ "mac":
{ "k": "Pz4lpSpJ3/qbgNkv9g5m9g==", "r": "eSzdCoRqKgUMWSwJGPylCg==" },
"encrypt": "BPlHYRVMNhu12MlJH5Zq91Lm1KH09UqVk0QRr2Jcblo="}"""

        result = restic.cat.masterkey()

        mock_execute.assert_called_with([
            'restic',
            '--json',
            'cat',
            'masterkey',
        ])

        self.assertEqual(
            {
                'mac': {
                    'k': 'Pz4lpSpJ3/qbgNkv9g5m9g==',
                    'r': 'eSzdCoRqKgUMWSwJGPylCg=='
                },
                'encrypt': 'BPlHYRVMNhu12MlJH5Zq91Lm1KH09UqVk0QRr2Jcblo='
            }, result)


class CatConfigTest(unittest.TestCase):

    @mock.patch.object(cat.command_executor, 'execute')
    def test_cat_config(self, mock_execute):
        mock_execute.return_value = """{ "version": 2,
"id": "dc0268fe690023237565f1ca58c257350ea5b86ffabc3a067933c271a3c0998a",
"chunker_polynomial": "2f7ca3c3c5dbad"}"""

        result = restic.cat.config()

        mock_execute.assert_called_with([
            'restic',
            '--json',
            'cat',
            'config',
        ])

        self.assertEqual(
            {
                'version':
                    2,
                'id':
                    'dc0268fe690023237565f1ca58c257350ea5b86ffabc3a067933c271a3c0998a',  # pylint: disable=C0301
                'chunker_polynomial':
                    '2f7ca3c3c5dbad'
            },
            result)



import json

from restic.internal import command_executor


def run(restic_base_command, pattern):
    cmd = restic_base_command + ['find', pattern]

    return json.loads(command_executor.execute(cmd))



from restic.internal import command_executor


def run(restic_base_command, from_repo=None, from_password_file=None):

    cmd = restic_base_command + ['copy']

    if from_repo:
        cmd.extend(['--from-repo', from_repo])

    if from_password_file:
        cmd.extend(['--from-password-file', from_password_file])

    return command_executor.execute(cmd)



from restic.internal import command_executor


def run(restic_base_command,
        snapshot_id='latest',
        include=None,
        exclude=None,
        target_dir=None):
    cmd = restic_base_command + ['restore', snapshot_id]

    if include:
        cmd.extend(['--include', include])

    if exclude:
        cmd.extend(['--exclude', exclude])

    if target_dir:
        cmd.extend(['--target', target_dir])

    return command_executor.execute(cmd)



import unittest
from unittest import mock

import restic
from restic.internal import init


class InitTest(unittest.TestCase):

    @mock.patch.object(init.command_executor, 'execute')
    def test_init_with_no_parameters(self, mock_execute):
        mock_execute.return_value = """
{"message_type":"initialized","id":"d0c84a66bffea61b4cbb88c39cea742127699b3f3af71127b68edcc142edff48","repository":"/tmp/tmp.lNaHYLGR7F"}
""".strip()

        repository_id = restic.init()

        self.assertEqual(
            'd0c84a66bffea61b4cbb88c39cea742127699b3f3af71127b68edcc142edff48',
            repository_id)
        mock_execute.assert_called_with(['restic', '--json', 'init'])

    # Prior to restic 0.15.0, restic responded with plaintext instead of JSON.
    @mock.patch.object(init.command_executor, 'execute')
    def test_init_legacy_format(self, mock_execute):
        mock_execute.return_value = (
            'created restic repository 054ed643d8 at /media/backup1')

        repository_id = restic.init()

        self.assertEqual('054ed643d8', repository_id)
        mock_execute.assert_called_with(['restic', '--json', 'init'])

    @mock.patch.object(init.command_executor, 'execute')
    def test_init_with_secondary_repository(self, mock_execute):
        mock_execute.return_value = """
{"message_type":"initialized","id":"d0c84a66bffea61b4cbb88c39cea742127699b3f3af71127b68edcc142edff48","repository":"/tmp/tmp.lNaHYLGR7F"}
""".strip()

        repository_id = restic.init(
            copy_chunker_params=True,
            from_repo='s3:https://some.backend.com/mybucket')

        self.assertEqual(
            'd0c84a66bffea61b4cbb88c39cea742127699b3f3af71127b68edcc142edff48',
            repository_id)
        mock_execute.assert_called_with([
            'restic', '--json', 'init', '--copy-chunker-params', '--from-repo',
            's3:https://some.backend.com/mybucket'
        ])



import json

from restic.internal import command_executor


def run(restic_base_command, snapshot_id=None, group_by=None, tags=None):
    cmd = restic_base_command + ['snapshots']

    if snapshot_id:
        cmd.append(snapshot_id)

    if group_by:
        cmd.extend(['--group-by', group_by])

    if tags:
        cmd.extend(['--tag', ','.join(tags)])

    return json.loads(command_executor.execute(cmd))
